package net.pricefx.workflowengine;

import net.pricefx.domain.User;
import net.pricefx.domain.UserGroup;
import net.pricefx.workflowengine.WorkflowStep.WorkflowStepExecutionStatus;
import net.pricefx.workflowengine.WorkflowStep.WorkflowStepType;

import java.util.*;

/**
 * 
 * This structure represents an entire workflow with multiple steps.
 * The steps are by definition sequential. Every step has more detailed info in it's own data object. 
 *
 *
 */
public class WorkflowInfo {

	/**
	 * The ordered steps of this workflow
	 */
	public ArrayList<WorkflowStepInfo> steps = new ArrayList<>();
	
	/**
	 * The currently active workflow step
	 */
	public WorkflowStepInfo activeStep;
	
	/**
	 * Extra data that is generated and filled by the workflow formula (optionally) to be used e.g. in email templates
	 */
	public Map<String, Object> dataMap = new LinkedHashMap<>();

	/**
	 * In case that the workflow was denied, and user selected any step (happens when
	 * WorkflowDTO.withSelectStepOnDeny(true) and user selected some state.
	 *
	 * Semantics of this is given by logic, system treats it as arbitratry data.
	 */
	public String selectedStepOnDeny;

	/**
	 * 
	 * This structure describes each workflow step in detail
	 *
	 */
	public static class WorkflowStepInfo {
		/**
		 * The unique workflow step identifier
		 */
		public String uniqueName;
		
		/**
		 * Is the step a an approval node by individual user(s)?
		 */
		public Boolean isUserApprover;
		
		/**
		 * Is this step an approval node by a user group?
		 */
		public Boolean isGroupApprover;
		
		/**
		 * A list of users that can approve
		 */
		public List<User> userApprovers;
		
		/** 
		 * A list of groups that can approve
		 */
		public List<UserGroup> userGroupApprovers;
		@Deprecated
		public String userGroupLabel;
		
		/**
		 * The status of the step
		 * Can be:
		 * INITIALIZED, PREEXECUTED, EXECUTED_DENIED, EXECUTED_APPROVED, EXECUTED_WITHDRAWN, ALREADY_SATISFIED
		 */
		public WorkflowStepExecutionStatus executionStatus;

		/**
		 * A workflow formula generated reason text to explain the step
		 */
		@Deprecated
		public String getReason() {
			return reasons != null && reasons.size() == 1 ? reasons.get(0) : null;
		}

		public List<String> reasons;

		/**
		 * Date of approval, GMT timezone
		 */
		public Date approvalDate;

		/**
		 * Date of approval, partition default timezone
		 */
		public Date getApprovalDatePartitionDefaultTZ() {
			return new Date();
		}
		
		/**
		 * A user comment entered during approval/denial
		 */
		public String comment;
		
		/**
		 * Either APPROVAL or WATCHER
		 * Watchers are just notified and do not need to act. Workflow advances immediately to next step
		 */
		public WorkflowStepType stepType;
		
		public Boolean isEmailWatcher;
		public Collection<String> watcherEmails;
		
		/**
		 * Has this step attachments to be sent along the notification email?
		 */
		public Boolean emailAttachment;

		/**
		 * If emailAttachment is true, then this settings allow to use any other than default excel template
		 */
		public String emailAttachmentTemplateName;
		
		/**
		 * The attachments
		 */
		public List<?> approvalRequiredEmailAttachments;
		
		/**
		 * Is the this step active?
		 */
		public Boolean isActiveStep;
		
		/**
		 * Approved by user(s)
		 */
		public List<User> executedByUsers;
		
		/**
		 * Approved by user(s) : user names for easier display
		 */
		public List<String> executedByNames;

		public Map<String, Object> stepHistory;

		@Deprecated
		public User getUserApprover() {
			return userApprovers != null && userApprovers.size() == 1 ? userApprovers.get(0) : null;
		}

		@Deprecated
		public UserGroup getUserGroupApprover() {
			return userGroupApprovers != null && userGroupApprovers.size() == 1 ? userGroupApprovers.get(0) : null;
		}
		
		public Boolean isApprovalStep() {
			return WorkflowStepType.APPROVAL.equals(stepType);
		}
		
		public Boolean isWatcherStep() {
			return WorkflowStepType.WATCHER.equals(stepType);
		}
		
		public Boolean isInitialized() {
			return WorkflowStepExecutionStatus.INITIALIZED.equals(executionStatus);
		}

		public Boolean isPreExecuted() {
			return WorkflowStepExecutionStatus.PREEXECUTED.equals(executionStatus);
		}

		public Boolean isDenied() {
			return WorkflowStepExecutionStatus.EXECUTED_DENIED.equals(executionStatus);
		} 

		public Boolean isApproved() {
			return WorkflowStepExecutionStatus.EXECUTED_APPROVED.equals(executionStatus);
		}

		public Boolean isAlreadySatisfied() {
			return WorkflowStepExecutionStatus.ALREADY_SATISFIED.equals(executionStatus);
		}
		
		public Boolean isWithdrawn() {
			return WorkflowStepExecutionStatus.EXECUTED_WITHDRAWN.equals(executionStatus);
		}
		
		public String getExecutionStatusI18n() {
			return executionStatus.i18n();
		}
		
	}
	
}
